A:link            { text-decoration: none; 
                    color: #0000FF;
                  }
A:visited         { text-decoration: none; 
                    color: #AA0088;
                  }
A:hover           { text-decoration: underline; 
                  }
A.banner          { color  : #FFFFFF;
                    font-size : small;
                    text-decoration : none;
                  }
BODY              { background-color: white; 
                    font-family: Verdana, sans-serif;
		    font-size: small;
                    margin-left: 10%; margin-right: 10%;
                 }
H1                { color: #0044FF;
                    font-family: Verdana, sans-serif; 
		    font-size: 20pt;
                    margin-left: -8%
                  }
H2                { color: #3366FF; 
                    font-family: Verdana, sans-serif; 
                    margin-left: -4%
                  }
H3                { color: #3366FF; 
                    font-family: Verdana, sans-serif; 
                    margin-left: -4%
                  }
H4                { color: #3366FF; 
                    font-family: Verdana, sans-serif; 
                    margin-left: -4%
                  }
INPUT             { background-color: white; }
OL                { font-family: Verdana, sans-serif; 
		    font-size: small;
		  }
UL                { font-family: Verdana, sans-serif; 
		    font-size: small;
	  	  }
LI                { font-family: Verdana, sans-serif; 
/*                    list-style: url(./images/reddot.gif) outside; */
                    margin-left: -10px;
                    list-style: circle;
		    font-size: small;
                  }
DL                { font-family: Verdana, sans-serif; 
		    font-size: small
		   }
DT                { font-family: Verdana, sans-serif;
 		    font-size: small;
		  }
DD                { font-family: Verdana, sans-serif; }

P                 { font-family: Verdana, sans-serif; 
		    font-size: small;
		  }
PRE.peptide       { font-size: small; }
PRE.dna           { font-size: small; 
                    font-family: monospace;
                  }
.projectrow       { background-color: white; }
SELECT            { background-color: white; }
TABLE             { padding: 0;
                    spacing: 2;
                    width: 100%;
                  }
TD                { font-family: Verdana, sans-serif; font-size: small }
TH                { font-family: Verdana, sans-serif; 
                    padding: 8; 
                  }
.codeblock        { color: black;
                    border: solid;
                    border-width: thin;
                    width: 100%;
                    font-size: medium;
                    background: #FFEEEE;
                  }
.highlight          { color: red;
                    border: solid;
                    border-width: thin;
                    width: 100%;
                    font-size: medium;
                    background: #FFFFFF;
                  }
.section-list     { color: black;
                    background: #EEEEFF;
                  }

.abstract         { background-color: #FFEEEE; }
.acetree          { font-size: 10pt; }
.banner           { font-size: x-large;
                    align: center;
                    background-color: blue;
                    color: yellow;
                    padding: 100px 3px;
                  }
.data             { font-size: small; }
.datatitle        { background-color: #93CBF4; }
.databody         { background-color: #CCFFFF; }
.search           { font-size: small; }
.searchbody       { background-color: #FFFFCC; }
.searchbodyalt    { background-color: #CCFFFF; }
.searchtitle      { background-color: #FFFF99; }
.description      { font-style: italic; }
.error            { color: red; }
                    color: #000000;
                  }
